function [ Trackmatrix ] = Find_birdtracks( Plotmatrix_c, v_min, v_max, v_tol, a_tol, N)
%FIND_BIRDTRACK_PLOTS 
%
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% Author: John-Olav Bratteb, 2014                      %
% Norwegian University of Science and Technology (NTNU) %
% jobrattebo@gmail.com                                  %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%
%{ 
Denne funksjonen tar inn plottmatrisen Plotmatrix og finner sett/tracks
med observerte plott som passer med realistiske fuglebevegelser. Disse
settene lagres i Trackmatrix p flgende form:
Trackmatrix
    Observert som nr    1   2   3   ...
Track 
1                       34  12  93  ...
2                       11  4   74  ...
3                       59  25  31  ...
..                      ..  ..  ..  ...

I eksempelet over vises 3 tracks, hvor den frste bestr av plott nr. 34,
s 12 og 93 (og forsetter kanskje med flere plott utover til hyre i 
matrisen), osv.
"Plott nr. 34" refererer til Plottet Plotmatrix(34,:)

v_min - Fuglenes antatte minste scan-til-scan hastighet i m/s
v_max - Fuglenes antatte strste scan-til-scan hastighet i m/s
v_tol - En fugls antatte strste forandring i hastighet over tre pflgende scans i m/s
a_tol - En fugls antatte strste forandring i retning fra scans[i,i+1] til scans[i+1,i+2] i grader

Ass_plots er en en-dimensjonal vektor som fungerer som en liste over hvilke
plott som er assosiert til et annet plott.

Pot_tracks er en liste over plott-assosiasjonene, dvs. plott som l
innenfor passe avstand fra hverandre ift v_min, v_max og forskjellen i
observasjonstidspunkt. Pot_tracks(4,1) = 23 betyr at plott nr.23 l
innenfor riktig avstand fra plott 4 til  kunne vre starten p et track.

Assmatrix er assosiasjonsmatrisen som nrmeste nabo (NN) bestemmes fra.
Assmatrix(1,:) er en liste over avstandene fra senter av gaten for 1 scan 
senere enn siste plott i tracket, til alle punkter som falt innenfor denne
gaten.Assmatrix(2,:) er det samme, men for scan nr 2 etter scannet det siste
plottet i tracket ble observert i (antar miss). Avstanden til alle punkter 
som ikke faller innenfor noen av de to gatene er Inf.
f.eks. Assmatrix(1,52)=12.39 betyr at plott 52 falt innenfor gate 1 og
ligger 12.39 meter fra den predikerte posisjonen som er senter av gate 1.
%}

%% Initialisering


Ass_plots = [];

Pot_tracks = zeros(size(Plotmatrix_c,1)); 
Trackmatrix = zeros(size(Plotmatrix_c,1),N);


%% Finn potensielle track startpunkt basert p scan 1, 2 og 3.
% Baserer seg p  finne punkter som ligger innenfor avstander fra
% hverandre som passer med fuglebevegelser.

for i = 1:2                                                      % for scan i
    for P = 1:size(Plotmatrix_c,1)                                 % for alle plot P i scan i
        % Sk etter punkter Q i neste scan som ligger mellom d_min og d_max fra P
        if Plotmatrix_c(P,1)~=0  && Plotmatrix_c(P,8) == i           % hvis plot P er observert i scan i
            for Q = 1:size(Plotmatrix_c,1)                         % for alle plot Q i scan i+1
                if Plotmatrix_c(Q,1)~=0 && Plotmatrix_c(Q,8) == i+1   
                    
                    if i==2 && ismember(Q,Ass_plots) % Hvis P er i scan 2, hopp over alle P som allerede er assosiert
                        continue
                    end

                    Px = Plotmatrix_c(P,1);
                    Py = Plotmatrix_c(P,2);
                    Qx = Plotmatrix_c(Q,1);
                    Qy = Plotmatrix_c(Q,2);

                    dPQ = sqrt( (Qx - Px)^2 + (Qy - Py)^2 ); % Avstand fra P til Q i meter

                    dtPQ = Plotmatrix_c(Q,3) - Plotmatrix_c(P,3);  % Forskjell i obs.tidspkt fra P til Q mlt i sekunder

                    d_min = v_min*dtPQ;
                    d_max = v_max*dtPQ;

                    if dPQ > d_min && dPQ < d_max % Hvis avstand mellom P og Q dPQ er mellom d_min og d_max
                        Ass_plots = [Ass_plots P Q];
                        Pot_tracks(P,find(Pot_tracks(P,:)==0,1,'first')) = Q; % Legg til Q i rad P i Pot_tracks
                        
                    end



                end
            end
        end

        % Hvis punkt P i frste scan ikke har ftt noen assosierte punkter
        % (miss i andre scan), let etter passende punkter i scan 3.
        if i == 1 && Plotmatrix_c(P,8) == 1 && find(Pot_tracks(P,:)==0,1,'first') == 1
            for Q = 1:size(Plotmatrix_c,1)                            % For alle plot Q i scan 3
                if Plotmatrix_c(Q,1)~=0 && Plotmatrix_c(Q,8) == i+2
                    Px = Plotmatrix_c(P,1);
                    Py = Plotmatrix_c(P,2);
                    Qx = Plotmatrix_c(Q,1);
                    Qy = Plotmatrix_c(Q,2);

                    dPQ = sqrt( (Qx - Px)^2 + (Qy - Py)^2 ); % Avstand fra P til Q i meter

                    dtPQ = Plotmatrix_c(Q,3) - Plotmatrix_c(P,3);  % Forskjell i obs.tidspkt fra P til Q mlt i sekunder
                    
                    d_min = v_min*dtPQ;
                    d_max = v_max*dtPQ;
                    
                    if dPQ > d_min && dPQ < d_max       % Hvis avstand mellom P og Q dPQ er mellom d_min og d_max
                        Ass_plots = [Ass_plots P Q];
                        Pot_tracks(P,find(Pot_tracks(P,:)==0,1,'first')) = Q; % Legg til Q i rad P i Pot_tracks
                    end
                end
            end
        end
        
        
    end
end

%% Track initsialisering 

for i = 1:find(Pot_tracks(:,1)~=0,1,'last')
    if Pot_tracks(i,1) ~= 0
        for j = 1:find(Pot_tracks(i,:)~=0,1,'last')
            Trackmatrix(find(Trackmatrix(:,1)==0,1,'first'),1:2) = [i Pot_tracks(i,j)];
        end
    end
end


%% Let videre etter tracks

for T = 1:size(Trackmatrix,1)           % For alle tracks T i Trackmatrix
    if Trackmatrix(T,1) ~= 0            
        for i = 1:size(Trackmatrix,2)-1   % For alle plott i track T
            if Trackmatrix(T,i) ~= 0 && Trackmatrix(T,i+1) ~= 0
                Assmatrix = Inf*ones(2,size(Plotmatrix_c,1)); %Initialiserer Assosieringsvektoren for Track T fra forelpig endepunkt Q
                
                
                P = Trackmatrix(T,i);    % P = plott i for track T
                Q = Trackmatrix(T,i+1);  % Q = plott i+1 for track T
                
                Px = Plotmatrix_c(P,1);
                Py = Plotmatrix_c(P,2);
                Qx = Plotmatrix_c(Q,1);
                Qy = Plotmatrix_c(Q,2);
                
                % Finn kurs relativ x-akse, positiv kning grader mot klokka
                PQx = Qx - Px;
                PQy = Qy - Py;

                PQbearing = (180/pi)*asin(PQy/sqrt(PQx^2 + PQy^2));
                if PQx < 0
                    PQbearing = 180 - PQbearing;
                elseif PQx > 0 && PQy < 0
                    PQbearing = 360 - abs(PQbearing);
                end
                
                % Finn fart
                dPQ = sqrt( (Qx - Px)^2 + (Qy - Py)^2 ); % Avstand fra P til Q i meter
                dtPQ = Plotmatrix_c(Q,3) - Plotmatrix_c(P,3);  % Forskjell i obs.tidspkt fra P til Q mlt i sekunder
                if dtPQ == 0
                    continue
                end
                    
                v = dPQ / dtPQ;             % Hastighet utledet av posisjonene til P og Q
                
                PQscandist = Plotmatrix_c(Q,8) - Plotmatrix_c(P,8); % Scanavstand fra P til Q
                
                % Predikterer posisjon R i scannet etter det Q ble observert i
                Rx = Qx + v*(dtPQ/PQscandist)*cos(PQbearing*pi/180);
                Ry = Qy + v*(dtPQ/PQscandist)*sin(PQbearing*pi/180);
                
                % Predikterer posisjon R2 i scann nr.2 etter det Q ble observert i
                R2x = Qx + 2*v*(dtPQ/PQscandist)*cos(PQbearing*pi/180);
                R2y = Qy + 2*v*(dtPQ/PQscandist)*sin(PQbearing*pi/180);
                
                % Setter Range Gate
                d_min = (v - v_tol)*dtPQ/PQscandist;
                d_max = (v + v_tol)*dtPQ/PQscandist;
                
                if d_min < v_min*dtPQ/PQscandist
                    d_min = v_min*dtPQ/PQscandist;
                end
                if d_max > v_max*dtPQ/PQscandist
                    d_max = v_max*dtPQ/PQscandist;
                end
                
                % Sk i alle plott S fra scannet etter det Q ble observert i 
                % og lagre avstanden fra R til S for alle plott S som faller 
                % innenfor gate'en.
                for S = 1:size(Plotmatrix_c,1)                                        % For alle plott S
                    if Plotmatrix_c(S,1)~=0  && ( Plotmatrix_c(S,8) == Plotmatrix_c(Q,8)+1 || Plotmatrix_c(S,8) == Plotmatrix_c(Q,8)+2 )
                        Sx = Plotmatrix_c(S,1);
                        Sy = Plotmatrix_c(S,2);
                        
                        dQS = sqrt( (Sx - Qx)^2 + (Sy - Qy)^2 ); % Avstand fra endepunkt Q til plott S
                        % dRS = sqrt( (Sx - R2x)^2 + (Sy - R2y)^2 ); % Avstand fra prediktert posisjon R2 til plott S
                        
                        % Leter etter punkter i Gate for scanavstand 1 fra Q
                        if d_min < dQS && dQS < d_max && Plotmatrix_c(S,8) == Plotmatrix_c(Q,8)+1 % Hvis S er innenfor Range Gate for scannet etter Q
                            % Finn vinklene til vektorene QR og QS
                            QRbearing = PQbearing;
                            QSx = Sx - Qx;
                            QSy = Sy - Qy;
                            QSbearing = (180/pi)*asin(QSy/sqrt(QSx^2 + QSy^2));
                            if QSx < 0
                                QSbearing = 180 - QSbearing;
                            elseif QSx > 0 && QSy < 0
                                QSbearing = 360 - abs(QSbearing);
                            end
                            
                            if ismember(1,abs(QRbearing-(QSbearing+[-360 0 360])) < a_tol)  % Hvis Plott S er innenfor Azimut Gate for 1 scanavstand
                                dRS = sqrt( (Sx - Rx)^2 + (Sy - Ry)^2 );        % Avstand fra prediktert posisjon R til plott S
                                Assmatrix(1,S) = dRS;     % Lagre avstanden dRS p S sin plass i assosieringsvektoren
                            end
                            
                        end
                        
                        
                        % Leter etter punkter i Gate for scanavstand 2 fra
                        % Q (miss i scannet etter Q)
                        if d_min*2 < dQS && dQS < d_max*2 && Plotmatrix_c(S,8) == Plotmatrix_c(Q,8)+2 % Hvis S er innenfor Range Gate for scan nr.2 etter Q
                            % Finn vinklene til vektorene QR og QS
                            QRbearing = PQbearing;
                            QSx = Sx - Qx;
                            QSy = Sy - Qy;
                            QSbearing = (180/pi)*asin(QSy/sqrt(QSx^2 + QSy^2));
                            if QSx < 0
                                QSbearing = 180 - QSbearing;
                            elseif QSx > 0 && QSy < 0
                                QSbearing = 360 - abs(QSbearing);
                            end
                            
                            if ismember(1,abs(QRbearing-(QSbearing+[-360 0 360])) < a_tol*2)  % Hvis Plott S er innenfor Azimut Gate for 1 scanavstand
                                dR2S = sqrt( (Sx - R2x)^2 + (Sy - R2y)^2 );        % Avstand fra prediktert posisjon R til plott S
                                Assmatrix(2,S) = dR2S;     % Lagre avstanden dRS p S sin plass i assosieringsvektoren
                            end
                        end
                    end
                end
                
                % Oppdater track T i Trackmatrix
                if ismember(1,Assmatrix(1,:)~=Inf)  % Hvis det er noen avstander lagret i Assmatrix for Gate i 1 scan etter Q
                    [~, NN] = min(Assmatrix(1,:)); % Finner nrmeste nabo (NN)
                    Trackmatrix(T,find(Trackmatrix(T,:)==0,1,'first')) = NN; % Lagrer NN som et nytt plott i track T
                
                elseif ismember(1,Assmatrix(2,:)~=Inf)  % Hvis det er noen avstander lagret i Assmatrix for Gate i scan nr.2 etter Q
                    [~, NN] = min(Assmatrix(2,:)); % Finner nrmeste nabo (NN)
                    Trackmatrix(T,find(Trackmatrix(T,:)==0,1,'first')) = NN; % Lagrer NN som et nytt plott i track T
                
                end
                
            end
        end
    end
end
% Strip trailing zeros in dim 1
Trackmatrix(Trackmatrix(:,1)==0,:)=[];
        
clearvars -except Trackmatrix    
            
          
end
