function [ Plotmatrix ] = GeneratePlotmatrix( Detmatrix )
%GENERATEPLOTMATRIX
%
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
% Author: John-Olav Bratteb, 2014                      %
% Norwegian University of Science and Technology (NTNU) %
% jobrattebo@gmail.com                                  %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
%
%{
% Funksjonen sker i deteksjonsmatrisen MTD_tot etter deteksjonshoper og
% genererer plotmatrisen Plotmatrix. 
% 
% Input: 
% Detmatrix er en deteksjonsmatrise som inneholder verdier 0-255, hvor 0 er
% ingen deteksjon, og verdi ~= 0 gir mlt dopplerskift. 127 = 0 doppler
% 
% Output:
% Plotmatrix er en plotmatrise p flgende form:
% Plotmatrix =  [range1, azimuth1, obs.tid1, doppler1, strrelse1, rangecelle1, PRI1;
%                range2, azimuth2, obs.tid2, doppler2, strrelse2, rangecelle2, PRI2;
%                    ... 
%                rangeN, azimuthN, obs.tidN, dopplerN, strrelseN, rangecelleN, PRIN;]
% 
% range og azimuth er her polarkoordinatene til et plot( basert p en 
% deteksjonshops tyngdepunkt). Obs.tid er observasjonstidspunktet for et 
% plot. Doppler er et plots dopplerverdi, og strrelse er antallet
% deteksjoner i MTD_tot som ble sltt sammen til plottet.
% 
% Potensiell effektivisering av GeneratePlotmatrix og find_detheap:
% - detheap trenger ikke inneholde koordinatene til alle deteksjonene hvis 
%   hopens tyngdepunkt returneres.
% - tyngdepunkt som center(r,az), ikke som to variabler.
% - prealloker detheap og Plot til gitt strrelse, alloker til "mer plass" 
%   med en fast blokkstrrelse ved behov.Husk i s fall  slette alle deler
%   av matrisene som ble allokert men ikke brukt.
%}

% tic
meters_per_range_cell = 1.25;
range_des_factor = 3;               % M settes lik det som settes i CNav
maxplots = 40000;                  % Maximum number of plots extracted
Plotmatrix = zeros(maxplots,7);
format long
[R, Az] = size(Detmatrix);
Checkedcells = zeros(R, Az);
Plotnr = 0;
global filename
header = double(Detmatrix(1:8,:));

%% Linerinterpoler asimut-retningstempel for hvert PRI i Detmatrix for kt
% nyaktighet
asimut = 256*header(1,:)+header(2,:);
asimut_inkr = [1, abs(diff(asimut))]; % Detekterer pri'ene der asimut inkrementeres og 
% utvider med en 1'er i starten for  f med det aller frste asimutspranget.
nord_hopp = asimut_inkr > 1; %Finner pri for spranget fra 359 grader til null.
asimut_inkr(nord_hopp) = 1; % Setter ogs dette spranget til +1.
% asimut_inkr inneholder n 1'ere p alle vinkelsprang, ellers er den null.
asimut_inkr = logical(asimut_inkr); % Caster fra float til logical.
asimut_subset = asimut(asimut_inkr); %Alle registrerte vinkler i fila, men kun et stk av hver.
pri_hele = 1:1:length(asimut);
pri_subset = pri_hele(asimut_inkr); %pri'ene der asimut inkrementerer.
asimut_interpol = interp1(pri_subset, asimut_subset, pri_hele, 'linear');

%Quickfix for NaNs in the last elements of asimut_interpol
nans = find(isnan(asimut_interpol));
if numel(nans)>0
    last_diff = asimut_interpol(nans(1)-1)-asimut_interpol(nans(1)-2);
    for i = nans
        asimut_interpol(i) = asimut_interpol(i-1) + last_diff;
    end
end

% Roter Nord med et bias for tilpasning til Nord i et kart
asimut_interpol = mod(asimut_interpol - 4,360);

%% Sker deteksjonsmatrisen MTD_tot etter deteksjoner
for i = 1:R;
    for j = 1:Az;
       if Detmatrix(i,j) ~= 0 && i > 5                          % Funn av deteksjon utenfor range-celle 5
           if Checkedcells(i,j) == 0                            % Hvis denne deteksjonen ikke er sjekket tidligere
               %% Ekstraher deteksjonshop               
               
               [Detheap, center_r, center_az, Checkedcells] = find_detheap(Detmatrix, i, j, Checkedcells);                    
               %Detheap = [det_r, det_az, det_doppler; 
               %           det_r2, det_az2, det_doppler2; 
               %                 ...                     ]

              
               %% Finn Time of Observation (TO)
               TO = (str2double(filename(end-8:end-4))-1 )* 2.333; % TO for frste puls i scan'et. TO = 0 er frste puls i frste fil. T = 2.333 er hentet fra snittsberegninger for fil 14069-14155. 
               % T = 2.333 s er usikker. For hele Case 1 (14069-14245) var
               % snittrotasjonstiden 2.345 s, alts 12 ms variasjon her.
               for p = 1:center_az                              % Summere opp dt for alle pulser p frem til Plottets senter i azimut (center_az)
                   TO = TO + (256*double(Detmatrix(3,p)) + double(Detmatrix(4,p)))*0.096*10^(-6);
               end
               
               %% Hent azimut-komponent fra header til azimut-celle/PRI,
               center_az_deg = asimut_interpol(center_az);  % asimut i grader med 0/360 grader i retning Nord med kende grader med klokka
               center_az_deg = mod(90-center_az_deg,360);   % asimut i grader med 0/360 grader i retning Hyre/st med kende grader mot klokka

               %% Center_r skal ikke telle med headeren p 8 elementer
               center_r_m = center_r-8;
               %% Center_r i meter
               center_r_m = meters_per_range_cell*range_des_factor*center_r_m;
               %% Finn plot_doppler
%                plot_doppler = Detmatrix(i,j); % Bruk doppler fra frste deteksjon funnet i detheap
               
               plot_doppler = 0;                                % Doppler-verdi basert p gjennomsnitt av komponenter i detheap
               for k = 1:size(Detheap,1)                        % Sl sammen denne lkken med den for beregning av tyngdepunkt hvis den blir brukt.
                   plot_doppler = plot_doppler + Detheap(k,3);
               end
               plot_doppler = plot_doppler / size(Detheap,1);
               
               %% Finn plotsize
               plotsize = size(Detheap,1);
               
               %% Oppdaterer Plotmatrisen med en plot
               Plotnr = Plotnr + 1;                             % Inkrementerer Plotnr for plot
               Plotmatrix(Plotnr,:) = [center_r_m, center_az_deg, TO, plot_doppler, plotsize, center_r, center_az];
               
               %% Har sjekket cellen
               Checkedcells(i,j) = 1;
           end
       end
    end
end

%% Plotting for evaluering
% figure()
% imagesc(Detmatrix)
% whitebg('white')
% % alpha(0.2)
% hold on
% scatter(Plotmatrix(:,7),Plotmatrix(:,6),200,'k','x','LineWidth',3)

%% Postprocess/cleanup
Plotmatrix = Plotmatrix(1:find(Plotmatrix(:,1)==0,1,'first')-1,:);    % Fjerner alle rekker med null
Plotmatrix = sortrows(Plotmatrix,3); % Sorter etter observasjonstidspunkt

clearvars -except Plotmatrix

% toc

end

